UserTeams = coopimport('code/user/teams.lua')

-- if one of these strings appears anywhere in the command string it is a cheat
cheatStrings = {
	'setfocusarmy',
	'createentity',
	'createunit',
	'copy',
	'add',
	'blingbling',
	'sallyshears',
	'sim',
	'execute',
	'teleport',
	'destroy',
	'kill',
	'ai_',
}

function IsCheat( cmd )
	-- returns false or the position a cheat was found (true)
	local cheatFound = false
	cmd = string.lower(cmd)
	for _,cheat in cheatStrings do
		cheatFound = string.find( cmd , cheat )
		if cheatFound then break end
	end
	return cheatFound
end

-- CATCH CONSOLE CHEATERS
local oldConExecute = ConExecute
function ConExecute( cmd )
	if IsCheat( cmd ) then
		print('Haa ha! You tried to cheat and I stopped you.')
	else
		oldConExecute( cmd )
	end
end

local oldConExecuteSave = ConExecuteSave
function ConExecuteSave( cmd )
	if IsCheat( cmd ) then
		print('Haa ha! You tried to cheat and I stopped you.')
	else
		oldConExecuteSave( cmd )
	end
end

-- STOP KEYBINDING CHEATERS
local oldIN_AddKeyMapTable = IN_AddKeyMapTable
function IN_AddKeyMapTable(keyMapTable)
	-- remove keys that activate cheats
	LOG('Clearing cheats from added keymap.')
	local newKeyMapTable = {}
	for key,data in keyMapTable do
		if IsCheat( data.action ) then
			LOG('Keybinding ['..key..'] = '..data.action..' removed because it contains a cheat.')
		else
			newKeyMapTable[key] = data
		end
	end
	oldIN_AddKeyMapTable( newKeyMapTable )
end

-- REMOVE DEBUG KEY CHEATS
local debugKeyMap = import('/lua/keymap/defaultKeyMap.lua').debugKeyMap
-- Remove all debug keys first
IN_RemoveKeyMapTable(debugKeyMap)
-- Now add them back with just the cheats removed
IN_AddKeyMapTable(debugKeyMap)


-- ONLY ALLOW FOCUS TO ALLIES
local lastArmy = nil
oldSetFocusArmy = SetFocusArmy
function SetFocusArmy( toArmy ) 
	LOG("COOP: SetFocusArmy called")
	
	-- Get the army they are in or the last army they were in
	if not lastArmy then lastArmy = GetFocusArmy() end
	fromArmy = GetFocusArmy()
	if fromArmy == -1 then fromArmy = lastArmy end
	
	-- Make sure team data is setup
	if not TeamData then
		error("COOP: Attempt to switch focus before team setup")
	end
	
	-- Do the switch
	if toArmy == -1 then
		-- Player relinquishing control, we must remember their army number so they can rejoin
		if fromArmy ~= -1 then
			LOG("COOP: Player gone AWOL")
			lastArmy = fromArmy
			oldSetFocusArmy(-1)
		end
		
	elseif toArmy == UserTeams.GetArmyLeader(fromArmy) then
		-- Joining team leader
		LOG("COOP: Setting focus to team leader")
		oldSetFocusArmy(toArmy)
		
	elseif UserTeams.IsAllied(fromArmy,toArmy) then
		-- Joining teammate
		LOG("COOP: Setting focus to teammate")
		oldSetFocusArmy(toArmy)
	else
		LOG("COOP: Cheat detected: You aren't on that team")
	end
end