-- COOP: Sim-side teams code
-- This code is a little more complex than you'd think because I want the team configurations to change dynamically if required.

data = {
	teams = {}, -- data stored by teamIndex
	armies = {}, -- data stored by armyIndex
	ready = false,
}
local default_team = {
	armies = {}, -- armies in team
	hasHumans = false, -- true if humans are on the team
}

-- setup team data
function InitTeams()
	LOG('COOP: Setting up team data')
	
	for name,army in ScenarioInfo.ArmySetup do
		--LOG(repr(name))
		--LOG(repr(army))
		
		-- Add army index to team list
		data.teams[army.Team] = data.teams[army.Team] or table.deepcopy(default_team)
		data.teams[army.Team].armies = data.teams[army.Team].armies or {}
		table.insert(data.teams[army.Team].armies,army.ArmyIndex)
		
		-- Set a team flag if there are any humans in it
		if army.Human then
			data.teams[army.Team].hasHumans = true
		end
		
		-- Add army to armies list
		data.armies[army.ArmyIndex] = army
		GetArmyLeader(army.ArmyIndex)
	end

	data.ready = true
end

-- Return true if army index or name is the leader of its team
function IsLeader(armyIndex)
	if type(armyIndex) == 'string' then
		armyIndex = ScenarioInfo.ArmySetup[armyIndex].ArmyIndex
	end
	return data.armies[armyIndex].Leader == armyIndex
end

-- Get the armyIndex of the leader of the team this army is in
function GetArmyLeader(armyIndex)
	local teamIndex = nil
	for name,army in ScenarioInfo.ArmySetup do
		if army.ArmyIndex == armyIndex then
			teamIndex = army.Team
			GetTeamLeader(teamIndex)
			break
		end
	end
	data.armies[armyIndex].Leader = data.teams[teamIndex].Leader
	return data.teams[teamIndex].Leader
end

-- Get the armyIndex of the leader of team
function GetTeamLeader(teamIndex)
	if not data.teams[teamIndex] or not data.teams[teamIndex].Leader then
		ChooseLeader(teamIndex)
	end
	--LOG('COOP: Army '..data.teams[teamIndex].Leader..' leads team '..teamIndex)
	return data.teams[teamIndex].Leader
end

-- Assign a leader to a team and store the result
function ChooseLeader(teamIndex)
	local best_pick = nil
	-- Pick first AI
	for name,army in ScenarioInfo.ArmySetup do
		if army.Team == teamIndex and not (army.Human or army.Civilian) then
			best_pick = army.ArmyIndex
			break
		end
	end
	-- otherwise choose first human
	if not best_pick then
		for name,army in ScenarioInfo.ArmySetup do
			if army.Team == teamIndex and army.Human then
				best_pick = army.ArmyIndex
				break
			end
		end
	end
	-- Cache result
	data.teams[teamIndex] = data.teams[teamIndex] or {}
	if best_pick then
		LOG('COOP: Army '..best_pick..' chosen to lead team '..teamIndex)
		data.teams[teamIndex].Leader = best_pick
		-- Update User/UI side
		Sync.TeamData = data
	end


end
