teams = coopimport('code/teams.lua')
scenarioUtils = import('/lua/sim/ScenarioUtilities.lua')

-- Override creation of initial units
oldAIBrain = AIBrain
AIBrain = Class(oldAIBrain) {
	
	OnSpawnPreBuiltUnits = function(self)
		-- COOP: Only spawn prebuilt stuff if we're the leader
		local thisArmyIndex = self:GetArmyIndex()
		local startX, startZ = self:GetArmyStartPos()
		if teams.IsLeader(thisArmyIndex) then
			LOG('COOP: Spawning prebuilt units for team leader')
			oldAIBrain.OnSpawnPreBuiltUnits(self)
			-- Spawn extra ACU's for teammates
			local offsetZ = 0 
			for name,army in teams.data.armies do
				if not army.Civilian and army.ArmyIndex ~= thisArmyIndex and army.Leader == thisArmyIndex then
					offsetZ = offsetZ + 3
					local factionIndex = GetArmyBrain(name):GetFactionIndex()
					local initialUnitName = import('/lua/factions.lua').Factions[factionIndex].InitialUnit
					local cdr = CreateUnitHPR(initialUnitName, thisArmyIndex, startX, 0, startZ + offsetZ, 0, 0, 0)
					LOG('COOP: '..army.PlayerName.. ' ACU spawned')
					cdr:SetCustomName(army.PlayerName)
				end
			end
		end
	end,
	
	BuildStructure = function(self, builder, structureName, locationInfo)
		-- Only build if we aren't busy
		local newLocation = {-1,-1,-1}
		local army = teams.data.armies[self:GetArmyIndex()]
		if not army.Human and teams.data.teams[army.Team].hasHumans then
			
			
			local lock = builder:GetLock()
			if lock then
				builder:SetCustomLabel('Building '..structureName..' locked by '..lock)
			else
				builder:SetCustomLabel('Building ' .. structureName)
				newLocation = oldAIBrain.BuildStructure(self, builder, structureName, locationInfo)
			end
		end
		return newLocation
	end,
	
	BuildUnit = function(self)
		-- COOP: Only spawn prebuilt stuff if we're the leader
		LOG('COOP: AI Building Unit')
		oldAIBrain.BuildUnit(self)
	end,
	
	MakePlatoon = function(self,arg1,aiName)
		-- COOP: We need to override some methods in the platoon to ensure locked units don't get orders
		LOG('Building Platoon '..arg1..' '..aiName)
		local platoon = oldAIBrain.MakePlatoon(self,arg1,aiName)
		LOG(string.sub(repr(platoon),1,500))
		local oldGetPlatoonUnits = platoon.GetPlatoonUnits
		platoon.GetPlatoonUnits = function(self)
			local units = oldGetPlatoonUnits(self)
			--LOG(string.sub(repr(units),1,100))
			return units
		end
		return platoon
	end,
}

--LOG(dump(AIBrain))
