-- COOP: All possible states a unit can be in (may be in more than one)
all_states = {
	'Immobile',
	'Moving',
	'Attacking',
	'Guarding',
	'Building',
	'Upgrading',
	'WaitingForTransport',
	'TransportLoading',
	'TransportUnloading',
	'MovingDown',
	'MovingUp',
	'Patrolling',
	'Busy',
	'Attached',
	'BeingReclaimed',
	'Repairing',
	'Diving',
	'Surfacing',
	'Teleporting',
	'Ferrying',
	'WaitForFerry',
	'AssistMoving',
	'PathFinding',
	'ProblemGettingToGoal',
	'NeedToTerminateTask',
	'Capturing',
	'BeingCaptured',
	'Reclaiming',
	'AssistingCommander',
	'Refueling',
	'GuardBusy',
	'ForceSpeedThrough',
	'UnSelectable',
	'DoNotTarget',
	'LandingOnPlatform',
	'CannotFindPlaceToLand',
	'BeingUpgraded',
	'Enhancing',
	'BeingBuilt',
	'NoReclaim',
	'NoCost',
	'BlockCommandQueue',
	'MakingAttackRun',
	'HoldingPattern',
	'SiloBuildingAmmo',
}

oldUnit = Unit
Unit = Class(oldUnit) {

	SetCustomName = function(self, name)
		-- Set the unit name, not just its label
		self.name = name
		oldUnit.SetCustomName(self, name)
	end,

	SetCustomLabel = function(self, name)
		-- Set the unit label only
		oldUnit.SetCustomName(self, name)
	end,
	
	CheckStateThread = function(self)
		while true do
			local state = self:GetStateString()
			local newname = ''
			if self.name ~= '' and state ~= '' then
				newname = self.name..': '..state
			elseif self.name ~= '' then
				newname = self.name
			else
				newname = self:GetStateString()
			end
			oldUnit.SetCustomName(self, newname)
			WaitSeconds(4.0)
		end
	end,
	
	OnCreate = function(self)
		oldUnit.OnCreate(self)
		self.name = ''
		self:ForkThread(self.CheckStateThread)
	end,
	
	Unlock = function(self)
		self:SetCustomName('Unlocked')
		self.locked = false
	end,

	Lock = function(self, name)
		self:SetCustomName('Locked for '..name)
		self.locked = true
	end,

	GetLock = function(self)
		local lock = nil 
		for _,stateName in all_states do
			if self:IsUnitState(stateName) then
				lock = stateName
				break
			end
		end
		return lock
	end,
	
	IsLocked = function(self)
		if self:GetLock() then
			return true
		else
			return false
		end
	end,

	GetStateString = function(self)
		-- Returns a comma-seperated list of current states or empty string if idle
		local name = nil
		for _,state in all_states do
			if self:IsUnitState(state) then
				if not name then
					name = state
				else
					name = name ..','.. state
				end
			end
		end
		return name or ''
	end,
	
	SetBusy = function(self, busy)
		if busy then
			--self:SetCustomName('Busy')
		else
			--self:SetCustomName('Idle')
		end
		oldUnit.SetBusy(self,busy)
	end,
	
	OnMotionHorzEventChange = function(self, new, old)
		--LOG('move new: '..new..' old: '..old)
		--self:SetCustomName(new)
		--LOG(repr(self))
		if new == 'Stopped' and not self:IsDead() then
			--self:Unlock()
		end
	end,
	
	IdleState = State {
		OnEnterState = function(self)
			--self:SetCustomName('Idle')
			self:SetBusy(false)
		end,
		
		OnExitState = function(self)
			--self:SetCustomName('Busy')
			self:SetBusy(true)
		end,
	}
}
